/*---------------- "exampleFragmentShader.frag" ----------------*/
// QUELLE vom Grundgerst: http://en.wikipedia.org/wiki/Vertex_Buffer_Object
// 
 
#version 150 // Specify which version of GLSL we are using.

precision highp float; 

struct LightSource
{ 
	vec3 color;
	vec3 position;
	float luminocity;

	float cAtt;
	float lAtt;
	float eAtt;

	vec2 near_far;

	
};

in vec3 position;
in vec2 uv;
in vec2 nUV;
in vec3 normal;
in vec3 normal_cameraspace;
in vec3 eye;

uniform float ambient;
uniform float percent_life;
uniform mat3 MV3x3;

out vec3 color;

uniform sampler2D myTextureSampler;
uniform sampler2D myNormalSampler;

uniform LightSource[8] lights;
uniform samplerCube[8] shadows;
in vec3 lightDirection_cameraspace[8];
in vec3 lightDirection_tangentspace[8];
in vec3 eyeDirection_tangentspace;

uniform int lightCount;

float shadow(const LightSource light, vec3 vertexPosition, vec2 near_far, int i){

	//Gaussian blur
	float intensities[9] = float[](	1/16f, 1/8f, 1/16f, 1/8f, 1/4f, 1/8f, 1/16f, 1/8f, 1/16f);

	float offset = 0.03f;
	vec3 vertexNeighbours[9] = vec3[](
	 vec3(vertexPosition.x - offset, vertexPosition.y - offset, vertexPosition.z), 
	 vec3(vertexPosition.x, vertexPosition.y - offset, vertexPosition.z),
	 vec3(vertexPosition.x + offset, vertexPosition.y - offset, vertexPosition.z), 

	 vec3(vertexPosition.x - offset, vertexPosition.y, vertexPosition.z),
	 vec3(vertexPosition.x, vertexPosition.y, vertexPosition.z), 
	 vec3(vertexPosition.x + offset, vertexPosition.y, vertexPosition.z),

	 vec3(vertexPosition.x - offset, vertexPosition.y + offset, vertexPosition.z), 
	 vec3(vertexPosition.x, vertexPosition.y + offset, vertexPosition.z),
	 vec3(vertexPosition.x + offset, vertexPosition.y + offset, vertexPosition.z));

	 float visibility = 1.0f;

	 for(int x = 0; x < 9; x++){

			vec3 vectorToLight = vertexNeighbours[x] - light.position;

			float smallest_dist_to_light = texture(shadows[i], vectorToLight).r;

			smallest_dist_to_light *= near_far.y; 

			float curr_dist_to_light = length(vectorToLight);

			if(smallest_dist_to_light + 0.15 < curr_dist_to_light){
				visibility -= intensities[x];
			}
	 }

	return visibility;
}


vec4 getLightColor(const LightSource light, int i, vec3 vertexPosition, vec3 vertexNormal, float ambient, vec3 eye, vec3 n)
{
	vec3 vectorToLight = vertexPosition - light.position;
	float dist = sqrt(length(vectorToLight));

	

	float visibility = 1.0;
	visibility = shadow(light, vertexPosition, light.near_far, i);

	vectorToLight = normalize(vectorToLight);
	vec3 E = normalize(vec3(0.0, 0.0, 0.0) - eye);
	vec3 l = normalize(lightDirection_tangentspace[i] );
	float cosTheta = clamp( dot( n,l ), 0,1 );
	vec3 R = reflect(l,n);
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	float attenuation = light.cAtt + light.lAtt*dist + light.eAtt*(dist*dist);


	return	vec4((texture2D(myTextureSampler, uv).xyz * visibility * light.color * light.luminocity * cosTheta / (dist*dist) +
			vec3(0.3,0.3,0.3) * light.color * visibility * light.luminocity * pow(cosAlpha,5) / (dist*dist)) * dist/attenuation, 1.0);


}

void main() 
{
		color = vec3(0,0,0);


	if(length(eye) < 20){
		vec3 vNormalized = normal_cameraspace;
		vec3 textureNormal_tangentspace = normalize(texture2D( myNormalSampler, vec2(uv.x,uv.y) ).rgb*2.0 - 1.0);

		vec4 texColor = texture2D(myTextureSampler, uv);
		vec3 n = textureNormal_tangentspace;

		vec4 lightColor = vec4(0);

		for(int i = 0; i < 8 && i < lightCount; i++){
			lightColor += getLightColor(lights[i], i, position, vNormalized, ambient, eye, n);
		}

		color = (texColor * ambient).xyz + (lightColor).xyz ;



		if(length(eye) > 18){
			color = ((20-length(eye))/2) * color;
		}

		color = color * vec3(1.3,1.3,1.6);
	}
	else{
		color = vec3(0,0,0);
	}


	vec3 colorNoLife = vec3(1, 0, 0);
	float life = min(1, percent_life + 0.4);
	color = (color * life + colorNoLife * (1- life));
	color = (color * vec3(2)) - vec3(0.1);
}
/*--------------------------------------------------------------*/