/*----------------- "exampleVertexShader.vert" -----------------*/
// QUELLE vom Grundgerst: http://en.wikipedia.org/wiki/Vertex_Buffer_Object
 
#version 150 // Specify which version of GLSL we are using.
#pragma optionNV(unroll all)
#pragma unroll
 
uniform mat4 M;
uniform mat4 V;
uniform mat4 P;
uniform sampler2D myTextureSampler;
uniform sampler2D myNormalSampler;
uniform int lightCount;
uniform mat3 MV3x3;

struct LightSource
{ 
	vec3 color;
	vec3 position;
	float luminocity;

	float cAtt;
	float lAtt;
	float eAtt;

};

uniform LightSource lights[8];
	
in vec3 vertexPosition;
in vec2 vertexUV;
in vec3 vertexNormal;
in vec3 vertexTangent;
in vec3 vertexBitangent;

out vec3 position;
out vec2 uv;
out vec3 normal;
out vec3 eye;
out vec2 nUV;
out vec3 normal_cameraspace;
out vec3 lightDirection_cameraspace[8];
out vec3 lightDirection_tangentspace[8];
out vec3 eyeDirection_tangentspace;

void main() 
{
	mat4 MVP = P * V * M;

	gl_Position = MVP * vec4(vertexPosition, 1.0f);

	position = vec3(M * vec4(vertexPosition, 1.0f));

	uv = vertexUV;
	nUV = vertexUV;

	normal = (M *vec4(vertexNormal, 1.0)).xyz;
	normal_cameraspace = ( V * M * vec4(vertexNormal,0)).xyz; 

	vec3 vertexPosition_cameraspace = (V * M * vec4(vertexPosition, 1.0f)).xyz;

	vec3 vertexNormal_cameraspace = MV3x3 * normalize(normal);
	vec3 vertexTangent_cameraspace = MV3x3 * normalize(vertexTangent);
	vec3 vertexBitangent_cameraspace = MV3x3 * normalize(vertexBitangent);

	mat3 TBN = transpose(mat3(
        vertexTangent_cameraspace,
        vertexBitangent_cameraspace,
        vertexNormal_cameraspace
    ));


	eye = (V * M * vec4(vertexPosition, 1.0f)).xyz;

	vec3 eyeDirection_cameraspace = vec3(0.0,0.0,0.0) - eye;

	for(int i = 0; i < 8 && i < lightCount; i++){
			vec3 lightPosition_cameraspace = ( V * vec4(lights[i].position,1)).xyz;
			lightDirection_cameraspace[i] = lightPosition_cameraspace + eyeDirection_cameraspace;

			lightDirection_tangentspace[i] = TBN * lightDirection_cameraspace[i];
			eyeDirection_tangentspace =  TBN * eyeDirection_cameraspace;

		}

}
/*--------------------------------------------------------------*/